﻿using System;
using System.Collections.Generic;
using System.Text;
using RGiesecke.DllExport;
using MSScriptControl;

namespace MSScript
{
    public class MSScript
    {
        // 1.0  initial release
        private static double version = 1.0;

        IScriptControl script;
        DScriptControlSource_Event dscript;


        static MSScript instance = new MSScript();

        string restype = "";
        bool errored = false;
        bool timedout = false;
        bool exceptioned = false;
        string exceptionmsg = "";
        // constructor

        protected MSScript()
        {
            script = new ScriptControl();
            dscript = (DScriptControlSource_Event)script;
            dscript.Error += ErrorEvent;
            dscript.Timeout += TimeoutEvent;
            script.Language = "VBScript";
            script.AllowUI = true;
            script.UseSafeSubset = false;
        }

        private static void clearerrors()
        {
            instance.errored = instance.timedout = instance.exceptioned = false;
            instance.exceptionmsg = "";
        }

        // private event handlers

        private static void ErrorEvent()
        {
            instance.errored = true;
        }

        private static void TimeoutEvent()
        {
            instance.timedout = true;
        }

        
        // Static methods

 
        [DllExport]
        public static String LibraryName()
        {
            return "MSScript.dll v" + version.ToString("0.00");
        }

        [DllExport]
        public static double LibraryVersion()
        {
            return version;
        }

        // public methods

        [DllExport]
        public static void About()
        {
            instance.script._AboutBox();
        }

        [DllExport]
        public static void AddCode(string scriptcode)
        {
            instance.script.AddCode(scriptcode);
        }

        // The Clear method is used to reset the Error object.
        // Using the AddCode, Eval, ExecuteStatement, or Reset methods will also clear the Error object before these methods begin processing. 
        [DllExport]
        public static void ClearError()
        {
            clearerrors();
            instance.script.Error.Clear();
        }

        [DllExport]
        public static string Eval(string expression)
        {
            try
            {
                clearerrors();
                object o = instance.script.Eval(expression);
                instance.restype = o.GetType().ToString();
                if (o == null)
                    o = "null";
                instance.restype = o.GetType().ToString();
                return o.ToString();
            }
            catch (Exception e)
            {
                instance.exceptioned = true;
                instance.exceptionmsg = e.Message;
                return "*Error*";
            }
       }

        [DllExport]
        public static void ExecuteStatement(string statement)
        {
            try
            {
                clearerrors();
                instance.script.ExecuteStatement(statement);
            }
            catch (Exception e)
            {
                instance.exceptioned = true;
                instance.exceptionmsg = e.Message;
            }
        }

        [DllExport]
        public static void Reset()
        {
            clearerrors();
            instance.script.Reset();
        }

        [DllExport]
        public static string Run(string startsub, string[] subparams)
        {
            try
            {
                clearerrors();
                instance.errored = instance.timedout = false;
                object[] p = new object[subparams.Length];
                Array.Copy(subparams, p, p.Length);
                object o = instance.script.Run(startsub, p);
                if (o == null)
                    o = "null";
                instance.restype = o.GetType().ToString();
                return o.ToString();
            }
            catch(Exception e)
            {
                instance.exceptioned = true;
                instance.exceptionmsg = e.Message;
                return "*Error*";
            }
        }



        // public properties


        [DllExport]
        public static bool Errored()
        { return instance.errored; }

        [DllExport]
        public static int ErrorColumn()
        { return instance.script.Error.Column; }

        [DllExport]
        public static string ErrorDescription()
        { return instance.script.Error.Description; }

        [DllExport]
        public static int ErrorLine()
        { return instance.script.Error.Line; }

        [DllExport]
        public static int ErrorNumber()
        { return instance.script.Error.Number; }

        [DllExport]
        public static string ErrorSource()
        { return instance.script.Error.Source; }

        [DllExport]
        public static string ErrorText()
        { return instance.script.Error.Text; }

        [DllExport]
        public static string ResultType()
        { return instance.restype; }


        [DllExport]
        public static void SetAllowUI(bool value)
        { instance.script.AllowUI = value; }

        // the maximum amount of time that a script can run before a warning message is displayed (if the AllowUI property is True)
        // –1 = no timeouts will occur.
        // 0 = the ScriptControl will monitor the execution of the script and will trigger the Timeout event if it determines that the script is hung.

        [DllExport]
        public static void SetScriptTimeout(int value)
        { instance.script.Timeout = value; }

        [DllExport]
        public static void SetUseSafeSubset(bool value)
        { instance.script.UseSafeSubset = value; }

        [DllExport]
        public static bool TimedOut()
        { return instance.timedout; }

        [DllExport]
        public static bool ExceptionFlag()
        { return instance.exceptioned; }

        [DllExport]
        public static string ExceptionMsg()
        { return instance.exceptionmsg; }




    }
}
